-- remove generated nodes if no link on it
create function ${model}._link_after_delete_fct()
returns trigger
language plpgsql
as $$$$
    begin
        delete from $model._node 
        where id=old.down 
        and generated is not null
        and (select count(1) from $model._link where up=old.down and link_type != 'street') = 0
        and (select count(1) from $model._link where down=old.down and link_type != 'street') = 0;
        delete from $model._node 
        where id=old.up 
        and generated is not null
        and (select count(1) from $model._link where up=old.up and link_type != 'street') = 0
        and (select count(1) from $model._link where down=old.up and link_type != 'street') = 0;
        return old;
    end;
$$$$
;;

create trigger ${model}_link_after_delete_trig
    after delete on $model._link
       for each row execute procedure ${model}._link_after_delete_fct()
;;


/* add trigger on reach geom update to recompute the position */
/* shall we update the pk according to the new position ? */
create function ${model}.reach_after_update_fct()
returns trigger
language plpgsql
as $$$$
    begin
        update $model.river_node set
            -- il faut valider cette mise à jour du pk
            geom = (select ST_LineInterpolatePoint(new.geom, ST_LineLocatePoint(new.geom, geom))) where reach=new.id;
        return new;
    end;
$$$$
;;

create trigger ${model}_reach_after_update_trig
    after update on $model.reach
       for each row execute procedure ${model}.reach_after_update_fct()
;;

/* add rule to create end points when reach is created, the pk are set automagically */
create function ${model}.reach_after_insert_fct()
returns trigger
language plpgsql
as $$$$
    begin
        insert into $model.river_node(geom) values(ST_StartPoint(new.geom));
        insert into $model.river_node(geom) values(ST_EndPoint(new.geom));
        return new;
    end;
$$$$
;;

create trigger ${model}_reach_after_insert_trig
    after insert on $model.reach
       for each row execute procedure ${model}.reach_after_insert_fct()
;;

/* add trigger on branch to update singularities validities */
create function ${model}.branch_after_fct()
returns trigger
language plpgsql
as $$$$
    begin
        if tg_op = 'INSERT' then
            update $model.pipe_branch_marker_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.marker_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.hydraulic_cut_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.zregul_weir_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.param_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.regul_sluice_gate_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.gate_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.borda_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            return new;
        else
            if tg_op = 'UPDATE' then
                update $model.pipe_branch_marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.hydraulic_cut_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.zregul_weir_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.param_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.regul_sluice_gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.borda_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                return new;
            else
                update $model.pipe_branch_marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.hydraulic_cut_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.zregul_weir_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.param_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.regul_sluice_gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.borda_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                return old;
            end if;
        end if;
    end;
$$$$
;;

create trigger ${model}_branch_trig
    after insert or update or delete on $model.branch
       for each row execute procedure ${model}.branch_after_fct()
;;

/* add trigger on reach to update singularities validities */
create function ${model}.reach_after_fct()
returns trigger
language plpgsql
as $$$$
    begin
        if tg_op = 'INSERT' then
            update $model.bradley_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.bridge_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.marker_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.hydraulic_cut_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.zregul_weir_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.param_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.regul_sluice_gate_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.gate_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            update $model.borda_headloss_singularity
                set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom));
            return new;
        else
            if tg_op = 'UPDATE' then
                update $model.bradley_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.bridge_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.hydraulic_cut_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.zregul_weir_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.param_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.regul_sluice_gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.borda_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(new.geom)) or st_intersects(geom, st_endpoint(new.geom))
                    or st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                return new;
            else
                update $model.bradley_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.bridge_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.marker_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.hydraulic_cut_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.zregul_weir_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.param_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.regul_sluice_gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.gate_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                update $model.borda_headloss_singularity
                    set geom=geom
                where st_intersects(geom, st_startpoint(old.geom)) or st_intersects(geom, st_endpoint(old.geom));
                return old;
            end if;
        end if;
    end;
$$$$
;;

create trigger ${model}_reach_trig
    after insert or update or delete on $model.reach
       for each row execute procedure ${model}.reach_after_fct()
;;



/* add rule affect catchemnt to catchment_node at creation */
create function ${model}.catchment_after_insert_fct()
returns trigger
language plpgsql
as $$$$
    begin
        update $model.catchment_node set contour=new.id where ST_Intersects($model.catchment_node.geom, new.geom);
        return new;
    end;
$$$$
;;

create trigger ${model}_catchment_after_insert_trig
    after insert on $model.catchment
       for each row execute procedure ${model}.catchment_after_insert_fct()
;;

/* trigger to update lien when the end points change */
create function ${model}.node_update_fct()
returns trigger
language plpgsql
as $$$$
    begin
        update $model._link set geom=ST_SetPoint(geom, 0, new.geom) where up=new.id;
        update $model._link set geom=ST_SetPoint(geom, ST_NumPoints(geom)-1, new.geom) where down=new.id;
        return new;
    end;
$$$$
;;

create trigger ${model}_node_update_trig
    after update of geom on $model._node
       for each row execute procedure ${model}.node_update_fct()
;;

/* validity rule for singularities : not at branch or reach extremities */
create function ${model}.on_branch_or_reach_endpoint(geom geometry)
returns boolean
language plpgsql
as $$$$
    declare
        res_ boolean;
    begin
        with forbidden as (select st_startpoint(reach.geom) as point from $model.reach
                               union
                               select st_endpoint(reach.geom) as point from $model.reach
                               union
                               select st_startpoint(branch.geom) as point from $model.branch
                               union
                               select st_endpoint(branch.geom) as point from $model.branch
                               )
        select exists(select 1 from forbidden where st_intersects(geom, forbidden.point)) into res_;
        return res_;
    end;
$$$$
;;

/*
@todo créer un trigger de vérification spécifique:

Restriction supplémentaire pour les objets RK, ZQ, ZT, ZF : s’ils sont placés sur une
branche filaire (reach ou branche de collecteur) ils doivent être connectés au nœud down de
la branche, car ils sont interprétés comme des conditions down.
*/

create function ${model}.check_hydrogramme_apport_on_down_route_fct()
returns boolean
language plpgsql
as $$$$
    declare
        res boolean;
    begin
        /* apres: routing with hydrograms at down */
        with routing_with_hydrogramme_apport_at_down as (
            select r.id as rid from $model._link as r, $model._hydrograph_bc_singularity as a
            where r.down=a.id and r.link_type=$routing_hydrology_link_type
        )
        select count(1)=0 from $model._link as r
        where r.down_type!=$manhole_hydrology_node_type and r.link_type=$routing_hydrology_link_type
        and r.id not in (select rid from routing_with_hydrogramme_apport_at_down) into res;
        return res;
    end;
$$$$
;;

create function ${model}.check_catchment_has_one_route_fct()
returns boolean
language plpgsql
as $$$$
    declare
        res boolean;
    begin
        with link as (
            select count(1) as count from $model.catchment_node as bv, $model._link as r
            where bv.id=r.up and r.link_type=$routing_hydrology_link_type
            group by r.id)
        select max(count)=min(count) and min(count)=1 from link into res;
        return res;
    end;
$$$$
;;

create view $model.connectivity_manhole as
    /*nb up is the count ok link for wich the node is down*/
    with up as (select n.id as amid, sum((r.down=n.id)::integer) as nb_up
        from $model._link as r, $model._node as n
        where r.link_type=$pipe_link_type
        group by n.id
    ),
    down as (select n.id as avid, sum((r.up=n.id)::integer) as nb_down
        from $model._link as r, $model._node as n
        where r.link_type=$pipe_link_type
        group by n.id
    )
    select n.id, n.geom, nb_up, nb_down
        from $model._node as n, up, down
        where n.id=amid and n.id=avid and n.node_type=$manhole_node_type
;;

create view $model.connectivity_manhole_hydrology as
    /*nb up is the count ok link for wich the node is down*/
    with up as (select n.id as amid, sum((r.down=n.id)::integer) as nb_up
        from $model._link as r, $model._node as n
        where r.link_type=$pipe_link_type
        group by n.id
    ),
    down as (select n.id as avid, sum((r.up=n.id)::integer) as nb_down
        from $model._link as r, $model._node as n
        where r.link_type=$pipe_link_type
        group by n.id
    )
    select n.id, n.geom, nb_up, nb_down
        from $model._node as n, up, down
        where n.id=amid and n.id=avid and n.node_type=$manhole_hydrology_node_type
;;

-- create function ${model}.check_clavh_is_down_fct()
-- returns boolean
-- language plpgsql
-- as $$$$
--     declare
--         res boolean;
--     begin
--         select coalesce(max(nb_down)=0, True) from $model.clavh_singularity as s, $model.connectivity_manhole_hydrology as c where c.id=s.id into res;
--         return res;
--     end;
-- $$$$


create index ${model}_elem_2d_node_contour_idx on $model._elem_2d_node using gist(contour)
;;


create function ${model}.line_set_z(
    in line geometry('LINESTRINGZ', $srid), 
    in z_up real, 
    in z_down real)
returns geometry('LINESTRINGZ', $srid)
language plpgsql
as $$$$
    declare
        geom geometry('LINESTRINGZ', $srid);
    begin
        with d as (select (ST_DumpPoints(line)).geom as geom),
        p as (select ST_X(d.geom) as x, ST_Y(d.geom) as y, 
            ST_LineLocatePoint(line, d.geom) as a
            from d),
        i as (select x, y, (1-a)*z_up + a*z_down as z, a from p)
        select ST_SetSRID(ST_Makeline(ST_MakePoint(x, y, z) order by a), $srid) as geom from i into geom;
        return geom;
    end;
$$$$
;;


create function ${model}.update_z()
returns boolean
language plpgsql
as $$$$
    begin

        update ${model}.river_node set geom=('SRID=$srid;POINTZ('||ST_X(geom)||' '||ST_Y(geom)||' '||z_ground||')')::geometry;

        update ${model}.pipe_link as p set geom=${model}.line_set_z(geom, z_invert_up, z_invert_down);


        -- find reach that are missing sections at end points, interpolate the sections

        /*
        -- the 
        with d as (select id, (ST_DumpPoints(geom)).geom as geom
            from ${model}.reach
            union
            select reach as id, geom 
            from ${model}.river_node),
        da as (select d.id, ST_X(d.geom) as x, ST_Y(d.geom) as y, ST_LineLocatePoint(l.geom, d.geom) as a
            from d join ${model}.reach as l on l.id=d.id),
        p as (select n.id, n.reach, t.z_invert_up, t.z_invert_down, ST_LineLocatePoint(r.geom, t.geom) as a
            from ${model}.river_node as n 
            join ${model}.river_cross_section_profile as t on n.id=t.id
            join ${model}.reach as r on n.reach=r.id),
        i as (select id, x, y, a, 
            (select max(a) from p where p.a <= da.a and p.reach=da.id) as au,
            (select min(a) from p where p.a >= da.a and p.reach=da.id) as ad
            from da),
        j as (select id, x, y, a, au, ad, (a-au)/(ad-au) as ratio 
            from i where ad > au
            union
            select id, x, y, a, au, ad, 1. as ratio from i where ad=au),
        k as (select id, x, y, 
            (select coalesce(z_invert_down, z_invert_up)*ratio + coalesce(z_invert_up, z_invert_down)*(1-ratio) from p where p.a=j.a) as z from j)
        select * from k;

        -- split the reach into measured segments
        -- give altitude to segments
        -- add discontinuties
        -- linemerge
        with s as (
            select ST_LineSubstring(r.l, u.a, d.a) as geom, 
            coalesce(u.z_invert_down, u.z_invert_up) as z_up,
            coalesce(u.z_invert_up, u.z_invert_down) as z_down

        )


        --river_cross_section_profile
        */

        return 't';
        
    end;
$$$$
;;

