create table $model._$tablename(
    id integer unique not null,
    link_type hydra_link_type not null,
        foreign key (id, link_type) references $model._link(id, link_type) on delete cascade,
        check(link_type=$tableid)$tabledeclare
);;

create view $model.$tablename as
    select p.id, p.name, p.up, p.down, $tableview, p.geom, p.configuration::varchar as configuration, p.generated, p.validity, p.up_type, p.down_type, p.configuration as  configuration_json
    from $model._$tablename as c, $model._link as p
    where p.id = c.id
;;

/* note: id and link_type are not updatable */
create function ${model}.${tablename}_fct()
returns trigger
language plpgsql
as $$$$
    declare
        id_ integer;
        up_ integer;
        up_type_ hydra_node_type;
        down_ integer;
        down_type_ hydra_node_type;
        dump_ boolean;
    begin
        if tg_op = 'INSERT' then
            select * from ${model}.find_link_fct(new.geom) into up_, up_type_, down_, down_type_;
            insert into $model._link(link_type, up, up_type, down, down_type, name, geom, configuration, generated)
                values($tableid, up_, up_type_, down_, down_type_, coalesce(new.name, 'define_later'), new.geom, new.configuration::json, new.generated) returning id into id_;
            update $model._link set name = (select abbreviation||id_::varchar 
                from hydra.link_type where name = $tableid) where name = 'define_later' and id = id_;
            insert into $model._$tablename 
                values (id_, $tableid, $tableinsert);
            if $tableid = $pipe_link_type then
                perform $model.branch_update_fct();
            end if;
            --perform $model.set_link_altitude(id_);
            perform $model.add_configuration_fct(new.configuration::json, id_, '$tablename');
            update $model._link set validity = (select $validity from  $model._$tablename where id = id_) where id = id_;
            new.id = id_;
            return new;
        elsif tg_op = 'UPDATE' then
            select * from ${model}.find_link_fct(new.geom) into up_, up_type_, down_, down_type_;
            update $model._link set up=up_, up_type=up_type_, down=down_, down_type=down_type_, name=new.name, geom=new.geom, configuration=new.configuration::json, generated=new.generated where id=old.id;
            ${commentupdate}update $model._$tablename set $tableupdate where id=old.id;
            if $tableid = $pipe_link_type then
               perform $model.branch_update_fct();
            end if;
            perform $model.add_configuration_fct(new.configuration::json, old.id, '$tablename');
            update $model._link set validity = (select $validity from  $model._$tablename where id = old.id) where id = old.id;
            return new;
        elsif tg_op = 'DELETE' then
            delete from $model._$tablename where id=old.id;
            delete from $model._link where id=old.id;
            if $tableid = $pipe_link_type then
               perform $model.branch_update_fct();
            end if;
            return old;
        end if;
    end;
$$$$;;

create trigger ${model}_${tablename}_trig
    instead of insert or update or delete on $model.$tablename
       for each row execute procedure ${model}.${tablename}_fct();;

