-- Regards de hydrologiques

create table $model._$tablename(
    id integer unique not null,
    node_type hydra_node_type not null,
        foreign key (id, node_type) references $model._node(id, node_type) on delete cascade,
        check(node_type=$tableid)$tabledeclare
)
;;

create view $model.$tablename as
    select p.id, p.name, $tableview, p.geom, p.configuration::varchar as configuration, p.generated, p.validity, p.configuration as configuration_json
    from $model._$tablename as c, $model._node as p
    where p.id = c.id
;;

/* note: id and node_type are not updatable */
create function ${model}.${tablename}_fct()
returns trigger
language plpgsql
as $$$$
    declare
        id_ integer;
    begin
        if tg_op = 'INSERT' then
            insert into $model._node(node_type, name, geom, configuration, generated)
                values($tableid, coalesce(new.name, 'define_later'), $constraingeom, new.configuration::json, new.generated) returning id into id_;
            update $model._node set name = (select abbreviation||id_::varchar 
                from hydra.node_type where name = $tableid) where name = 'define_later' and id = id_;
            insert into $model._$tablename 
                values (id_, $tableid, $tableinsert);
            perform $model.add_configuration_fct(new.configuration::json, id_, '$tablename');
            update $model._node set validity = (select $validity from  $model._$tablename where id = id_) where id = id_;
            new.id = id_;
            return new;
        elsif tg_op = 'UPDATE' then
            update $model._node set name=new.name, geom=$constraingeom, configuration=new.configuration::json, generated=new.generated where id=old.id;
            ${commentupdate}update $model._$tablename set $tableupdate where id=old.id;
            perform $model.add_configuration_fct(new.configuration::json, old.id, '$tablename');
            update $model._node set validity = (select $validity from  $model._$tablename where id = old.id) where id = old.id;
            return new;
        elsif tg_op = 'DELETE' then
            delete from $model._$tablename where id=old.id;
            delete from $model._node where id=old.id;
            return old;
        end if;

    end;
$$$$
;;

create trigger ${model}_${tablename}_trig
    instead of insert or update or delete on $model.$tablename
       for each row execute procedure ${model}.${tablename}_fct()
;;


