create table $model._river_cross_section_profile(
    id integer unique not null references $model._river_node(id),
    name varchar(24) unique not null,
    z_invert_up real,
    z_invert_down real,
    type_cross_section_up hydra_cross_section_type,
    type_cross_section_down hydra_cross_section_type,
    up_rk real,
    up_rk_maj real,
    up_sinuosity real,
    up_circular_diameter real,
    up_ovoid_height real,
    up_ovoid_top_diameter real,
    up_ovoid_invert_diameter real,
    up_cp_geom integer references $model.closed_parametric_geometry(id),
    up_op_geom integer references $model.open_parametric_geometry(id),
    up_vcs_geom integer references $model.valley_cross_section_geometry(id),
    up_vcs_topo_geom integer references $model.valley_cross_section_topo_geometry(id),
    down_rk real,
    down_rk_maj real,
    down_sinuosity real,
    down_circular_diameter real,
    down_ovoid_height real,
    down_ovoid_top_diameter real,
    down_ovoid_invert_diameter real,
    down_cp_geom integer references $model.closed_parametric_geometry(id),
    down_op_geom integer references $model.open_parametric_geometry(id),
    down_vcs_geom integer references $model.valley_cross_section_geometry(id),
    down_vcs_topo_geom integer references $model.valley_cross_section_topo_geometry(id),
    validity bool
)
;;

create view $model.river_cross_section_profile as
    select g.id,
        c.name,
        c.z_invert_up,
        c.z_invert_down,
        c.type_cross_section_up,
        c.type_cross_section_down,
        c.up_rk,
        c.up_rk_maj,
        c.up_sinuosity,
        c.up_circular_diameter,
        c.up_ovoid_height,
        c.up_ovoid_top_diameter,
        c.up_ovoid_invert_diameter,
        c.up_cp_geom,
        c.up_op_geom,
        c.up_vcs_geom,
        c.up_vcs_topo_geom,
        c.down_rk,
        c.down_rk_maj,
        c.down_sinuosity,
        c.down_circular_diameter,
        c.down_ovoid_height,
        c.down_ovoid_top_diameter,
        c.down_ovoid_invert_diameter,
        c.down_cp_geom,
        c.down_op_geom,
        c.down_vcs_geom,
        c.down_vcs_topo_geom,
        c.validity,
        g.geom 
    from $model._river_cross_section_profile as c, $model._node as g
    where g.id = c.id
;;


-- insert and update 
create function ${model}.river_cross_section_profile_fct()
returns trigger
language plpgsql
as $$$$
    begin
        select (
            (new.type_cross_section_up is null or (
                new.z_invert_up is not null
                and (new.up_rk is not null and new.up_rk >0)
                and (new.type_cross_section_up!='circular' or (new.up_circular_diameter is not null and new.up_circular_diameter > 0))                
                and (new.type_cross_section_up!='ovoid' or (new.up_ovoid_top_diameter is not null and new.up_ovoid_top_diameter > 0))
                and (new.type_cross_section_up!='ovoid' or (new.up_ovoid_invert_diameter is not null and new.up_ovoid_invert_diameter > 0))
                and (new.type_cross_section_up!='ovoid' or (new.up_ovoid_height is not null and new.up_ovoid_height >0 ))
                and (new.type_cross_section_up!='ovoid' or (new.up_ovoid_height > (new.up_ovoid_top_diameter+new.up_ovoid_invert_diameter)/2))
                and (new.type_cross_section_up!='pipe' or new.up_cp_geom is not null)
                and (new.type_cross_section_up!='channel' or new.up_op_geom is not null)
                and (new.type_cross_section_up!='valley' or (new.up_rk_maj is not null and new.up_rk_maj >0))
                and (new.type_cross_section_up!='valley' or (new.up_sinuosity is not null and new.up_sinuosity >0))
                and (new.type_cross_section_up!='valley' or new.up_vcs_geom is not null)
                )
            )
            and
            (new.type_cross_section_down is null or (
                new.z_invert_down is not null
                and (new.down_rk is not null and new.down_rk >0)
                and (new.type_cross_section_down!='circular' or (new.down_circular_diameter is not null and new.down_circular_diameter > 0))                
                and (new.type_cross_section_down!='ovoid' or (new.down_ovoid_top_diameter is not null and new.down_ovoid_top_diameter > 0))
                and (new.type_cross_section_down!='ovoid' or (new.down_ovoid_invert_diameter is not null and new.down_ovoid_invert_diameter > 0))
                and (new.type_cross_section_down!='ovoid' or (new.down_ovoid_height is not null and new.down_ovoid_height >0 ))
                and (new.type_cross_section_down!='ovoid' or (new.down_ovoid_height > (new.down_ovoid_top_diameter+new.down_ovoid_invert_diameter)/2))
                and (new.type_cross_section_down!='pipe' or new.down_cp_geom is not null)
                and (new.type_cross_section_down!='channel' or new.down_op_geom is not null)
                and (new.type_cross_section_down!='valley' or (new.down_rk_maj is not null and new.down_rk_maj >0))
                and (new.type_cross_section_down!='valley' or (new.down_sinuosity is not null and new.down_sinuosity >0))
                and (new.type_cross_section_down!='valley' or new.down_vcs_geom is not null)
                )
            )
            and
            (new.type_cross_section_up is null or new.type_cross_section_down is null or (
                new.z_invert_up is not null and new.z_invert_down is not null 
                )
            )
        ) into new.validity;
        if tg_op = 'INSERT' then
            /* find the node if not specified */
            select id from $model.river_node where ST_DWithin(new.geom, geom, .1) into new.id;
            select coalesce(new.name, 'CP'||new.id::varchar) into new.name;
            if new.id is null then
                raise exception 'no river node nearby river_cross_section_profile %', st_astext(new.geom);
            end if;
            insert into $model._river_cross_section_profile
                values (
                    new.id,
                    new.name,
                    new.z_invert_up,
                    new.z_invert_down,
                    new.type_cross_section_up,
                    new.type_cross_section_down,
                    new.up_rk,
                    new.up_rk_maj,
                    new.up_sinuosity,
                    new.up_circular_diameter,
                    new.up_ovoid_height,
                    new.up_ovoid_top_diameter,
                    new.up_ovoid_invert_diameter,
                    new.up_cp_geom,
                    new.up_op_geom,
                    new.up_vcs_geom,
                    new.up_vcs_topo_geom,
                    new.down_rk,
                    new.down_rk_maj,
                    new.down_sinuosity,
                    new.down_circular_diameter,
                    new.down_ovoid_height,
                    new.down_ovoid_top_diameter,
                    new.down_ovoid_invert_diameter,
                    new.down_cp_geom,
                    new.down_op_geom,
                    new.down_vcs_geom,
                    new.down_vcs_topo_geom,
                    new.validity
                );
            return new;
        elsif tg_op = 'UPDATE' then
            update $model._river_cross_section_profile set
                name=new.name,
                z_invert_up=new.z_invert_up,
                z_invert_down=new.z_invert_down,
                type_cross_section_up=new.type_cross_section_up,
                type_cross_section_down=new.type_cross_section_down,
                up_rk=new.up_rk,
                up_rk_maj=new.up_rk_maj,
                up_sinuosity=new.up_sinuosity,
                up_circular_diameter=new.up_circular_diameter,
                up_ovoid_height=new.up_ovoid_height,
                up_ovoid_top_diameter=new.up_ovoid_top_diameter,
                up_ovoid_invert_diameter=new.up_ovoid_invert_diameter,
                up_cp_geom=new.up_cp_geom,
                up_op_geom=new.up_op_geom,
                up_vcs_geom=new.up_vcs_geom,
                up_vcs_topo_geom=new.up_vcs_topo_geom,
                down_rk=new.down_rk,
                down_rk_maj=new.down_rk_maj,
                down_sinuosity=new.down_sinuosity,
                down_circular_diameter=new.down_circular_diameter,
                down_ovoid_height=new.down_ovoid_height,
                down_ovoid_top_diameter=new.down_ovoid_top_diameter,
                down_ovoid_invert_diameter=new.down_ovoid_invert_diameter,
                down_cp_geom=new.down_cp_geom,
                down_op_geom=new.down_op_geom,
                down_vcs_geom=new.down_vcs_geom,
                down_vcs_topo_geom=new.down_vcs_topo_geom,
                validity=new.validity
            where id=old.id;
            return new;
        end if;
    end;
$$$$
;;

create trigger ${model}_river_cross_section_profile_trig
    instead of insert or update on $model.river_cross_section_profile
       for each row execute procedure ${model}.river_cross_section_profile_fct()
;;

-- delete
create function ${model}.river_cross_section_profile_del_fct()
returns trigger
language plpgsql
as $$$$
    begin
        delete from $model._river_cross_section_profile where id=old.id;
        return old;
    end;
$$$$
;;

create trigger ${model}_river_cross_section_profile_del_trig
    instead of delete on $model.river_cross_section_profile
       for each row execute procedure ${model}.river_cross_section_profile_del_fct()
;;

create table $model._river_cross_section_pl1d(
    id integer unique not null references $model._river_node(id),
    name varchar(24) unique not null,
    profile geometry('LINESTRINGZ',$srid) not null check(ST_NPoints(profile)=4 and ST_IsValid(profile)),
    generated integer references $model.generation_step(id) on delete cascade
)
;;

create view $model.river_cross_section_pl1d as
    select g.id, c.name, c.profile, c.generated, g.geom 
    from $model._river_cross_section_pl1d as c, $model._node as g
    where g.id = c.id
;;

create function ${model}.river_cross_section_pl1d_fct()
returns trigger
language plpgsql
as $$$$
    declare
        id_ integer;
    begin
        if tg_op = 'INSERT' then
            /* find the node if not specified */
            select id from $model.river_node where ST_DWithin(new.profile, geom, .1) into id_;
            if id_ is null then
                raise exception 'no river node nearby river_cross_section_pl1d %', st_astext(new.geom);
            end if;
            insert into $model._river_cross_section_pl1d
                values (id_, coalesce(new.name, 'PL1D'||id_::varchar), new.profile, new.generated);
            new.id = id_;
            return new;
        elsif tg_op = 'UPDATE' then
            update $model._river_cross_section_pl1d set name=new.name, profile=new.profile, generated=new.generated where id=old.id;
            return new;
        elsif tg_op = 'DELETE' then
            delete from $model._river_cross_section_pl1d where id=old.id;
            return old;
        end if;
    end;
$$$$
;;

create trigger ${model}_river_cross_section_pl1d_trig
    instead of insert or update or delete on $model.river_cross_section_pl1d
       for each row execute procedure ${model}.river_cross_section_pl1d_fct()
;;



