create table $model._$tablename(
    id integer unique not null,
    singularity_type hydra_singularity_type not null,
        foreign key (id, singularity_type) references $model._singularity(id, singularity_type),
        check(singularity_type=$tableid)$tabledeclare
);;

create view $model.$tablename as
    select p.id, p.name, n.id as node, $tableview, n.geom, p.configuration::varchar as configuration, p.validity, p.configuration as configuration_json
    from $model._$tablename as c, $model._singularity as p, $model._node as n
    where p.id = c.id and n.id = p.node

;;

create function ${model}.${tablename}_fct()
returns trigger
language plpgsql
as $$$$
    declare
        id_ integer;
        nid_ integer;
        node_type_ hydra_node_type;
        point_ geometry('POINTZ',$srid);
    begin
        if tg_op = 'INSERT' then
            /* find the node if not specified */
            select id, node_type, geom from $model._node where ST_DWithin(new.geom, geom, .1) order by ST_Distance(new.geom, geom) limit 1 into nid_, node_type_, point_;
            if nid_ is null then
                raise exception 'singularity % not on node % (distance %)', ST_AsText(new.geom), (select name from $model._node ORDER BY geom <-> new.geom limit 1), (select ST_Distance(new.geom, geom) from $model._node ORDER BY geom <-> new.geom limit 1);
            end if;
            insert into $model._singularity(node, node_type, singularity_type, name, configuration)
                values (nid_, node_type_, $tableid, coalesce(new.name, 'define_later'), new.configuration::json) returning id into id_;
            update $model._singularity set name = (select abbreviation||id_::varchar 
                from hydra.singularity_type where name = $tableid) where name = 'define_later' and id = id_;

            insert into $model._$tablename
                values (id_, $tableid, $tableinsert);
            if $tableid = $pipe_branch_marker_singularity_type then
                perform $model.branch_update_fct();
            end if;
            perform $model.add_configuration_fct(new.configuration::json, id_, '$tablename');
            update $model._singularity set validity = (select $validity from  $model._$tablename where id = id_) where id = id_;
            new.id = id_;
            return new;
        elsif tg_op = 'UPDATE' then
            /* find the node if not specified */
            select id, node_type, geom from $model._node where ST_DWithin(new.geom, geom, .1) order by ST_Distance(new.geom, geom) limit 1 into nid_, node_type_, point_;
            if nid_ is null then
                raise exception 'singularity % not on node % (distance %)', ST_AsText(new.geom), (select name from $model._node ORDER BY geom <-> new.geom limit 1), (select ST_Distance(new.geom, geom) from $model._node ORDER BY geom <-> new.geom limit 1);
            end if;
            update $model._singularity set node=nid_, node_type=node_type_, name=new.name, configuration=new.configuration::json where id=old.id;
            ${commentupdate}update $model._$tablename set $tableupdate where id=old.id;
            if $tableid = $pipe_branch_marker_singularity_type then
                perform $model.branch_update_fct();
            end if;
            perform $model.add_configuration_fct(new.configuration::json, old.id, '$tablename');
            update $model._singularity set validity = (select $validity from  $model._$tablename where id = old.id) where id = old.id;
            return new;
        elsif tg_op = 'DELETE' then
            delete from $model._$tablename where id=old.id;
            delete from $model._singularity where id=old.id;
            if $tableid = $pipe_branch_marker_singularity_type then
                perform $model.branch_update_fct();
            end if;
            return old;
        end if;
    end;
$$$$;;

create trigger ${model}_${tablename}_trig
    instead of insert or update or delete on $model.$tablename
       for each row execute procedure ${model}.${tablename}_fct();;

