create table project.dry_inflow_sector(
    id serial primary key,
    name varchar(24) not null default project.unique_name('SECT_'),
    comment varchar(256),
    geom geometry('POLYGON',$srid) not null check(ST_IsValid(geom))
)
;;

create table project.dry_inflow_hourly_modulation(
    id serial primary key,
    name varchar(24) not null default project.unique_name('MODUL_'),
    comment varchar(256),
    hv_array real[][] check(array_length(hv_array, 2)=2)
)
;;

create table project.dry_inflow_scenario(
    id serial primary key,
    name varchar(24) not null default project.unique_name('DRYSCN_'),
    comment varchar(256)
)
;;

create table project.dry_inflow_scenario_sector_setting(
    dry_inflow_scenario integer not null references project.dry_inflow_scenario(id) on delete cascade,
    sector integer not null references project.dry_inflow_sector(id) on delete cascade,
        unique(dry_inflow_scenario, sector),
    vol_curve integer references project.dry_inflow_hourly_modulation(id),
    volume_sewage_m3day real not null default 0,
    coef_volume_sewage_m3day real not null default 0,
    volume_clear_water_m3day real not null default 0,
    coef_volume_clear_water_m3day real not null default 0
)
;;

create table project._rainfall(
    id serial primary key,
    rainfall_type hydra_rainfall_type not null,
        unique(id, rainfall_type),
    name varchar(24) not null,
    comment varchar(256),
    validity boolean
)
;;

create table project.coef_montana(
    id serial primary key,
    name varchar(24) not null default project.unique_name('MONTANA_'),
    comment varchar(256),
    return_period_yr real not null,
    coef_a real not null check(coef_a > 0),
    coef_b real not null check( coef_b < 0)
)
;;

