create table project._$tablename(
    id integer unique not null,
    rainfall_type hydra_rainfall_type not null,
        foreign key (id, rainfall_type) references project._rainfall(id, rainfall_type),
        check(rainfall_type=$tableid)$tabledeclare
);;

create view project.$tablename as
    select p.id, $tableview, p.name 
    from project._$tablename as c, project._rainfall as p
    where p.id = c.id

;;

/* note: id and rainfall_type are not updatable */
create function project.${tablename}_fct()
returns trigger
language plpgsql
as $$$$
    declare
        id_ integer;
    begin
        if tg_op = 'INSERT' then
            insert into project._rainfall(rainfall_type, name)
                values($tableid, coalesce(new.name, 'define_later')) returning id into id_;
            update project._rainfall set name = 'RAIN_'||id_::varchar
                where name = 'define_later' and id = id_;
            insert into project._$tablename 
                values (id_, $tableid, $tableinsert);
            update project._rainfall set validity = (select $validity from  project._$tablename where id = id_) where id = id_;
            new.id = id_;
            return new;
        elsif tg_op = 'UPDATE' then
            update project._rainfall set name=new.name where id=old.id;
            ${commentupdate}update project._$tablename set $tableupdate where id=old.id;
            update project._rainfall set validity = (select $validity from  project._$tablename where id = old.id) where id = old.id;
            return new;
        elsif tg_op = 'DELETE' then
            delete from project._$tablename where id=old.id;
            delete from project._rainfall where id=old.id;
            return old;
        end if;

    end;
$$$$
;;

create trigger project_${tablename}_trig
    instead of insert or update or delete on project.$tablename
       for each row execute procedure project.${tablename}_fct()
;;


