from multicorn import ForeignDataWrapper
import numpy

class W15ForeignDataWrapper(ForeignDataWrapper):

    def __init__(self, options, columns):
        ForeignDataWrapper.__init__(self, options, columns)
        self.__columns = columns
        f = open(options['file'], 'rb')

        f.seek(0)
        nb_step, nb_nod, nb_lias = numpy.frombuffer(f.read(32), dtype=numpy.int32)[:3]
        name_idx = {}
        value_map = []
        names = []
        type_ids = []
        assert(nb_lias == 0)
        for i in range(nb_nod):
            name = f.read(24).decode('utf-8').rstrip()
            name_idx[name] = i
            itype, id_ =  numpy.frombuffer(f.read(8), dtype=numpy.int32)[:2]
            type_ids.append(itype)
            if itype > 0 and itype < 7:
                value_map.append({
                        1: ['q_up', 'z_up', 'v_min', 'deb'],
                        2: ['q_up', 'z_up', 'z_down', 'deb_over_qmaj', 'hdepot', 'qs'],
                        3: ['q_up'],
                        4: ['z', 'vol', '', '', 'hdepot', 'qs'],
                        5: ['q_up', 'z_up', 'z_down', 'zs_over_zb', 'hdepot', 'qs'],
                        6: ['z', 'vol', '', '', 'hdepot', 'qs'],
                        7: ['z', 'vol', 'u_x', 'u_y', 'hdepot', 'qs'],
                        }[itype])
            else:
                value_map.append([])
            names.append(name)

        self.__name_idx = name_idx
        self.__file = f
        self.__nb_step = nb_step
        self.__nb_nod = nb_nod
        self.__value_map = value_map
        self.__names = names
        self.__type_ids = type_ids

    def __del(self):
        self.__file.close()

    def __read(self, step, idx):
        self.__file.seek(32*(1 + self.__nb_nod)*(1 + step) + 32*idx)
        return numpy.frombuffer(self.__file.read(32), dtype=numpy.float32)

    def execute(self, quals, columns):
        steps = range(self.__nb_step)
        names = None
        for qual in quals:
            if 'step' == qual.field_name:
                if qual.operator == '=':
                    steps = [qual.value]
            elif 'name' == qual.field_name:
                if qual.operator == '=':
                    names = [qual.value]

        indices = range(self.__nb_nod) \
                if names is None else\
                sorted([self.__name_idx[name] for name in names])

        for step in steps:
            for idx in indices:
                values = self.__read(step, idx)
                map_ = self.__value_map[idx]
                yield {column: (values[map_.index(column)]
                        if column in map_ else
                        (self.__names[idx] if column == 'name' else
                        (step if column == 'step' else
                        (idx if column == 'idx' else
                        (self.__type_ids[idx] if column == 'itype' else
                        None)))))
                        for column in columns}
